#!/usr/bin/perl
#
# V1.1
#
# Now tries a little bit harder in Step 0 to tidy up the long
# lines, so Step 1 can just wrap them

my $DEBUG = 0;

#
#
# Step 0: Read in the file and do a bit of pre mangling
#
#

my $alldata = "";

if ( ! $ARGV[0] ) {
	die "Please specific an input .asc file";
}

open(FILE, $ARGV[0]);

$/ = "\r";

while(<FILE>) {
	chomp;
	my $line = $_;

	# Remove spaces from the start.
	$line =~ s/^ +//g;

	# Just trim comments
	if ( ( length( $line ) >= 31 ) && ( $line =~ /^\// ) ) {
		$line = sprintf( "%.31s", $line );
	}
	# Split voice lines
	if ( length( $line ) >= 32 ) {
		# It seems to be safe to split on 'V(oice)' lines
		$line =~ s/(V\d)/\n$1/g;
	}

	# And hope everything else wraps okay

	# Lower case throws things

	$line =~ tr/a-z/A-Z/;
	$alldata .= $line . "\n";
}
close(FILE);

#
#
# Step 1: Read the data again, and translate the file
# into the internal data format
#
#

my $orchdata = "";
my $orchlength = 0;
my @blocks;
my $w = 0;

# Now read and translate each character.

foreach ( split//, $alldata ) {

	my $c = $_;
	my $a = ord($c);

	# Comment char - not sure why this needs mapping,
	# but it does

	if ( $c =~ /\// ) {
		$a = 111;
	}

	# These files have no 'newlines', but simply set
	# the 8th bit of the character to show it's the start
	# of the line.

	if( $first ) {
		$a += 128;
	} 

	# Lowercase characters and *some* others have the 7th bit set.

	if ( ( $a != 13 ) && ( $a != 10 ) && (  $c !~ /[\/A-Z\@_]/ ) ) {
		$a +=64;
	}

	# If the file has a new line, set the flag so we know to
	# add 128 to the next character.

	if ( ( ord($c) == 10 ) || ( ord($c) == 13 ) ) {
		$first = 1;
		$a = 0;
		$w = 0;
	} else {
		$first = 0;
	}

	if ( $a ) {
		$w++;

		# Hopefully we've caught all the long lines
		# but if not, just truncate. 99/100 it's a comment
		# anyway.
		#
		# Bad logic T! Letting them line wrap now

		#if ( $w >= 32 ) { next; }

		$orchdata .= chr($a);

		if ( $DEBUG ) {
			# Watch the translations go by 

			print $w . ": " . $c . '[' . ord($c) . ']  = ' . $a;
			my $d = <STDIN>;
		}

		# Every 255 characters, make a block ready for 
		# sending to tape

		if ( length( $orchdata ) == 255 ) {
			push @blocks, $orchdata;
			warn $#blocks . " blocks loaded";
			$orchdata = "";
			$orchlength += 255;
		}
	}
}
close(FILE);

# Load last block

push @blocks, $orchdata;
warn $#blocks . " blocks loaded";
$orchlength += length( $orchdata );
warn "Length will be " . $orchlength;

#
#
# Step 2: Prepare a tape
#
#

# Okay, first build a leader

my $output = chr(0x55)x16;

# A header block, data block or EOF file block consists of:
# 1. A leader byte - $55 2. A sync byte - $3C 3. A block type byte:
# 00=namefile block

$output .= chr(0x55) . chr(0x3c);

#
#
# Namefile Block
#
#

my $namefile =  chr(0x00);
# A block length byte (0-255) (in this case, 15)
$namefile .= chr(0x0f);

# Namefile Block:

my $name = $ARGV[0];
$name =~ /(.*)\./;
$name = $1;
$name =~ tr/a-z/A-Z/;
$name =~ tr/^[0-9|A-Z]//;
$name = sprintf( "%-8.8s", $name );

#5.1 An 8 byte program name
$namefile .= $name;
#    5.2 A file ID byte where:
#        00=BASIC program
#        01=Data file
#        02=Binary file
$namefile .= chr(0x01);
#    5.3 An ASCII flag where:
#        00=Binary file
#        FF=ASCII file
$namefile .= chr(0xff);
#    5.4 A gap flag to indicate whether the
#        data stream is continuous (00) as
#        in binary or BASIC files, or in blocks
#        where the tape keeps stopping (FF) as
#        in data files.
$namefile .= chr(0x00);
#    5.5 Two bytes for the default EXEC address
#        of a binary file.
# I think Orch-90 uses this to store the length of the 
# composition
$namefile .= chr( int( $orchlength / 256 ) ) . chr( $orchlength % 256 );
#    5.6 Two bytes for the default load address
#        of a binary file.
$namefile .= chr(0x00) . chr(0x00);
#6. A checksum byte which is: sum of all the data bytes + block type + block length.
my $sum = 0;
foreach my $ascval (unpack("C*", $namefile)) {
    $sum += $ascval;
}
$namefile .= chr( $sum % 256 );
#7. A trailer byte - $55
$namefile .= chr(0x55);
$namefile .= chr(0x55);

$output .= $namefile;

#
#
# Data Block
#
#

# Loop round the blocks of orch data we got earlier

foreach $orchdata ( @blocks ) {

# Header
	$output .= chr(0x55) . chr(0x3c);

	my $datablock =  chr(0x01);

# A block length byte (0-255) (in this case, 15)
	$datablock .= chr( length( $orchdata) );
	$datablock .= $orchdata;

#6. A checksum byte which is: sum of all the data bytes + block type + block length.
	my $sum = 0;
	foreach my $ascval (unpack("C*", $datablock)) {
		$sum += $ascval;
	}
	$datablock .= chr( $sum % 256 );

#7. A trailer byte - $55
	$datablock .= chr(0x55);
	$datablock .= chr(0x55);

	$output .= $datablock;
}

#
# END Block
#

$output .= chr(0x55) . chr(0x3c) . chr(0xff) . chr(0x0) . chr(0xff) . chr(0x55);

print $output;

